#include "avr/io.h"
#include "util/delay.h"
#include "avr/interrupt.h"
#include "uart.h"
#include "rprintf.h"

#define TASTER	PB1
#define TASTER_GEDRUECKT()	(PINB & (1<<TASTER))

#define LEDAN 	PORTD|=(1<<5)	
#define LEDAUS  PORTD&=(~(1<<5))

#define _MESSAGE_WELCOME_ "Daniel Cagara RadioControl v0.1\nAwaiting Commands through UART\n"


uint8_t led;
uint8_t refresh=0;

// Interrupt based UART Command Buffers
volatile char uartRecvString[125];
volatile uint8_t positionCnt=0;
volatile uint8_t dirty=0;

void voidTxBufferWrapper(u08 data)
{
	uartAddToTxBuffer(data);
}

void executeCommand(char* command)
{
	// Check for LED Command ON
	if(strcmp(command,"LEDON") == 0)
	{
		LEDAN;
		rprintf("LED IS ON NOW\n");
		dirty=1;
	}
	// Check for LED Command ON
	else if(strcmp(command,"LEDOFF") == 0)
	{
		LEDAUS;
		rprintf("LED IS OFF NOW\n");
		dirty=1;
	}
	// Otherwise we have an unknown command
	else
	{
		rprintf("Unknown Command, I don't know what to do\n");
		dirty=1;
	}

}

void setRecvStringEmpty()
{
	positionCnt=0;
	int i=0;
	for(i=0;i<125;i++)
		uartRecvString[i]=0;
}
    
void uartRxHandler(unsigned char c)
{
	if(c!=0x0D){ // If no new line char, then fill uart recv buffer!
		uartRecvString[positionCnt]=c; 
		positionCnt++;
	} else {
		rprintf("Received Command: ");
		rprintfStr(uartRecvString); // send welcome message
		rprintf("\n");
		executeCommand(&(uartRecvString[0]));
		dirty=1;
		setRecvStringEmpty();
	}
} 


void initializeUart()
{
	uartInit();                 // initialize UART (serial port)
    uartSetBaudRate(9600);      // set UART speed to 9600 baud
	rprintfInit(voidTxBufferWrapper);  // configure rprintf to use UART for output
	setRecvStringEmpty();
	uartSetRxHandler(uartRxHandler);
    rprintfStr(_MESSAGE_WELCOME_); // send welcome message
	dirty=1;
}

void initializePorts()
{
DDRD  |= (1 << 5); // Set Ports here, example: PORTD5 = LED, set as output
}


void mainLoop()
{
	while(1)
	{
		if(dirty==1){ // send any buffers out, and set dirty flag back to zero,
			uartSendTxBuffer();
			dirty=0;
		}
	}
}


int main(){
	initializePorts();
	initializeUart();
	mainLoop();
}
